/**
 * Fork of [dts-dom](https://github.com/RyanCavanaugh/dts-dom) with some additional features.
 *
 * dts-dom is licensed as follows:
 *
 * """
 *   Apache License
 *   Version 2.0, January 2004
 *   http://www.apache.org/licenses/
 *   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION
 *   1. Definitions.
 *   "License" shall mean the terms and conditions for use, reproduction, and distribution as defined by Sections 1 through 9 of this document.
 *   "Licensor" shall mean the copyright owner or entity authorized by the copyright owner that is granting the License.
 *   "Legal Entity" shall mean the union of the acting entity and all other entities that control, are controlled by, or are under common control with that entity. For the purposes of this definition, "control" means (i) the power, direct or indirect, to cause the direction or management of such entity, whether by contract or otherwise, or (ii) ownership of fifty percent (50%) or more of the outstanding shares, or (iii) beneficial ownership of such entity.
 *   "You" (or "Your") shall mean an individual or Legal Entity exercising permissions granted by this License.
 *   "Source" form shall mean the preferred form for making modifications, including but not limited to software source code, documentation source, and configuration files.
 *   "Object" form shall mean any form resulting from mechanical transformation or translation of a Source form, including but not limited to compiled object code, generated documentation, and conversions to other media types.
 *   "Work" shall mean the work of authorship, whether in Source or Object form, made available under the License, as indicated by a copyright notice that is included in or attached to the work (an example is provided in the Appendix below).
 *   "Derivative Works" shall mean any work, whether in Source or Object form, that is based on (or derived from) the Work and for which the editorial revisions, annotations, elaborations, or other modifications represent, as a whole, an original work of authorship. For the purposes of this License, Derivative Works shall not include works that remain separable from, or merely link (or bind by name) to the interfaces of, the Work and Derivative Works thereof.
 *   "Contribution" shall mean any work of authorship, including the original version of the Work and any modifications or additions to that Work or Derivative Works thereof, that is intentionally submitted to Licensor for inclusion in the Work by the copyright owner or by an individual or Legal Entity authorized to submit on behalf of the copyright owner. For the purposes of this definition, "submitted" means any form of electronic, verbal, or written communication sent to the Licensor or its representatives, including but not limited to communication on electronic mailing lists, source code control systems, and issue tracking systems that are managed by, or on behalf of, the Licensor for the purpose of discussing and improving the Work, but excluding communication that is conspicuously marked or otherwise designated in writing by the copyright owner as "Not a Contribution."
 *   "Contributor" shall mean Licensor and any individual or Legal Entity on behalf of whom a Contribution has been received by Licensor and subsequently incorporated within the Work.
 *   2. Grant of Copyright License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable copyright license to reproduce, prepare Derivative Works of, publicly display, publicly perform, sublicense, and distribute the Work and such Derivative Works in Source or Object form.
 *   3. Grant of Patent License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable (except as stated in this section) patent license to make, have made, use, offer to sell, sell, import, and otherwise transfer the Work, where such license applies only to those patent claims licensable by such Contributor that are necessarily infringed by their Contribution(s) alone or by combination of their Contribution(s) with the Work to which such Contribution(s) was submitted. If You institute patent litigation against any entity (including a cross-claim or counterclaim in a lawsuit) alleging that the Work or a Contribution incorporated within the Work constitutes direct or contributory patent infringement, then any patent licenses granted to You under this License for that Work shall terminate as of the date such litigation is filed.
 *   4. Redistribution. You may reproduce and distribute copies of the Work or Derivative Works thereof in any medium, with or without modifications, and in Source or Object form, provided that You meet the following conditions:
 *   You must give any other recipients of the Work or Derivative Works a copy of this License; and
 *   You must cause any modified files to carry prominent notices stating that You changed the files; and
 *   You must retain, in the Source form of any Derivative Works that You distribute, all copyright, patent, trademark, and attribution notices from the Source form of the Work, excluding those notices that do not pertain to any part of the Derivative Works; and
 *   If the Work includes a "NOTICE" text file as part of its distribution, then any Derivative Works that You distribute must include a readable copy of the attribution notices contained within such NOTICE file, excluding those notices that do not pertain to any part of the Derivative Works, in at least one of the following places: within a NOTICE text file distributed as part of the Derivative Works; within the Source form or documentation, if provided along with the Derivative Works; or, within a display generated by the Derivative Works, if and wherever such third-party notices normally appear. The contents of the NOTICE file are for informational purposes only and do not modify the License. You may add Your own attribution notices within Derivative Works that You distribute, alongside or as an addendum to the NOTICE text from the Work, provided that such additional attribution notices cannot be construed as modifying the License. You may add Your own copyright statement to Your modifications and may provide additional or different license terms and conditions for use, reproduction, or distribution of Your modifications, or for any such Derivative Works as a whole, provided Your use, reproduction, and distribution of the Work otherwise complies with the conditions stated in this License.
 *   5. Submission of Contributions. Unless You explicitly state otherwise, any Contribution intentionally submitted for inclusion in the Work by You to the Licensor shall be under the terms and conditions of this License, without any additional terms or conditions. Notwithstanding the above, nothing herein shall supersede or modify the terms of any separate license agreement you may have executed with Licensor regarding such Contributions.
 *   6. Trademarks. This License does not grant permission to use the trade names, trademarks, service marks, or product names of the Licensor, except as required for reasonable and customary use in describing the origin of the Work and reproducing the content of the NOTICE file.
 *   7. Disclaimer of Warranty. Unless required by applicable law or agreed to in writing, Licensor provides the Work (and each Contributor provides its Contributions) on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied, including, without limitation, any warranties or conditions of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are solely responsible for determining the appropriateness of using or redistributing the Work and assume any risks associated with Your exercise of permissions under this License.
 *   8. Limitation of Liability. In no event and under no legal theory, whether in tort (including negligence), contract, or otherwise, unless required by applicable law (such as deliberate and grossly negligent acts) or agreed to in writing, shall any Contributor be liable to You for damages, including any direct, indirect, special, incidental, or consequential damages of any character arising as a result of this License or out of the use or inability to use the Work (including but not limited to damages for loss of goodwill, work stoppage, computer failure or malfunction, or any and all other commercial damages or losses), even if such Contributor has been advised of the possibility of such damages.
 *   9. Accepting Warranty or Additional Liability. While redistributing the Work or Derivative Works thereof, You may choose to offer, and charge a fee for, acceptance of support, warranty, indemnity, or other liability obligations and/or rights consistent with this License. However, in accepting such obligations, You may act only on Your own behalf and on Your sole responsibility, not on behalf of any other Contributor, and only if You agree to indemnify, defend, and hold each Contributor harmless for any liability incurred by, or claims asserted against, such Contributor by reason of your accepting any such warranty or additional liability.
 *   END OF TERMS AND CONDITIONS
 * """
 */
export interface DeclarationBase {
    jsDocComment?: JsDocCommentDeclaration[];
    comment?: CommentDeclaration;
    flags?: DeclarationFlags;
    namespace?: NamespaceDeclaration;
}
export interface NamedDeclarationBase extends DeclarationBase {
    name: string;
}
export interface CommentDeclaration {
    kind: 'comment';
    comment: string;
    flags: CommentFlags;
}
export interface JsDocCommentDeclaration {
    kind: 'jsdoc-comment';
    jsDocComment: string;
    flags: CommentFlags;
}
export interface EnumMemberDeclaration extends DeclarationBase {
    kind: 'enum-value';
    name: string;
    value?: string | number;
}
export interface EnumDeclaration extends DeclarationBase {
    kind: 'enum';
    name: string;
    members: EnumMemberDeclaration[];
    constant: boolean;
}
export interface PropertyDeclaration extends DeclarationBase {
    kind: 'property';
    name: string;
    type: Type;
}
export interface Parameter {
    kind: 'parameter';
    name: string;
    type: Type;
    flags?: ParameterFlags;
}
export interface TypeParameter {
    kind: 'type-parameter';
    name: string;
    baseType?: ObjectTypeReference | TypeParameter;
}
export interface IndexSignature extends DeclarationBase {
    kind: 'index-signature';
    name: string;
    indexType: 'string' | 'number';
    valueType: Type;
}
export interface CallSignature extends DeclarationBase {
    kind: 'call-signature';
    parameters: Parameter[];
    returnType: Type;
    typeParameters: TypeParameter[];
}
export interface MethodDeclaration extends DeclarationBase {
    kind: 'method';
    name: string;
    parameters: Parameter[];
    returnType: Type;
    typeParameters: TypeParameter[];
}
export interface FunctionDeclaration extends DeclarationBase {
    kind: 'function';
    name: string;
    parameters: Parameter[];
    returnType: Type;
    typeParameters: TypeParameter[];
}
export interface ConstructorDeclaration extends DeclarationBase {
    kind: 'constructor';
    parameters: Parameter[];
}
export interface ClassDeclaration extends DeclarationBase {
    kind: 'class';
    name: string;
    members: ClassMember[];
    implements: InterfaceDeclaration[];
    typeParameters: TypeParameter[];
    baseType?: ObjectTypeReference;
}
export interface InterfaceDeclaration extends DeclarationBase {
    kind: 'interface';
    name: string;
    members: ObjectTypeMember[];
    baseTypes?: ObjectTypeReference[];
}
export interface ImportAllDeclaration extends DeclarationBase {
    kind: 'importAll';
    name: string;
    from: string;
}
export interface ImportNamedDeclaration extends DeclarationBase {
    kind: 'importNamed';
    names: Array<{
        name: string;
        as?: string;
    }>;
    from: string;
}
export interface ImportDefaultDeclaration extends DeclarationBase {
    kind: 'importDefault';
    name: string;
    from: string;
}
export interface ImportEqualsDeclaration extends DeclarationBase {
    kind: 'import=';
    name: string;
    from: string;
}
export interface ImportDeclaration extends DeclarationBase {
    kind: 'import';
    from: string;
}
export interface NamespaceDeclaration extends DeclarationBase {
    kind: 'namespace';
    name: string;
    members: NamespaceMember[];
}
export interface ConstDeclaration extends DeclarationBase {
    kind: 'const';
    name: string;
    type: Type;
}
export interface VariableDeclaration extends DeclarationBase {
    kind: 'var';
    name: string;
    type: Type;
}
export interface ExportEqualsDeclaration extends DeclarationBase {
    kind: 'export=';
    target: string;
}
export interface ExportDefaultDeclaration extends DeclarationBase {
    kind: 'exportDefault';
    name: string;
}
export interface ExportNameDeclaration extends DeclarationBase {
    kind: 'exportName';
    name: string;
    as?: string;
}
export interface ModuleDeclaration extends DeclarationBase {
    kind: 'module';
    name: string;
    members: ModuleMember[];
}
export interface ObjectType {
    kind: 'object';
    members: ObjectTypeMember[];
}
export interface UnionType {
    kind: 'union';
    members: Type[];
}
export interface IntersectionType {
    kind: 'intersection';
    members: Type[];
}
export interface FunctionType {
    kind: 'function-type';
    parameters: Parameter[];
    returnType: Type;
}
export interface TypeAliasDeclaration extends DeclarationBase {
    kind: 'alias';
    name: string;
    type: Type;
    typeParameters: TypeParameter[];
}
export interface ArrayTypeReference {
    kind: 'array';
    type: Type;
}
export interface NamedTypeReference {
    kind: 'name';
    name: string;
    typeParameters: TypeParameter[];
}
export interface TypeofReference {
    kind: 'typeof';
    type: NamedTypeReference;
}
export interface StringLiteral {
    kind: 'string-literal';
    value: string;
}
export interface NumberLiteral {
    kind: 'number-literal';
    value: number;
}
export interface TripleSlashReferencePathDirective {
    kind: 'triple-slash-reference-path';
    path: string;
}
export interface TripleSlashReferenceTypesDirective {
    kind: 'triple-slash-reference-types';
    types: string;
}
export interface TripleSlashReferenceNoDefaultLibDirective {
    kind: 'triple-slash-reference-no-default-lib';
    value: boolean;
}
export interface TripleSlashAmdModuleDirective {
    kind: 'triple-slash-amd-module';
    name?: string;
}
export declare type PrimitiveType = 'string' | 'number' | 'boolean' | 'any' | 'void' | 'object' | 'null' | 'undefined' | 'true' | 'false' | StringLiteral | NumberLiteral;
export declare type ThisType = 'this';
export declare type TripleSlashDirective = TripleSlashReferencePathDirective | TripleSlashReferenceTypesDirective | TripleSlashReferenceNoDefaultLibDirective | TripleSlashAmdModuleDirective;
export declare type TypeReference = TopLevelDeclaration | NamedTypeReference | ArrayTypeReference | PrimitiveType;
export declare type ObjectTypeReference = ClassDeclaration | InterfaceDeclaration | NamedTypeReference;
export declare type ObjectTypeMember = PropertyDeclaration | MethodDeclaration | IndexSignature | CallSignature;
export declare type ClassMember = PropertyDeclaration | MethodDeclaration | IndexSignature | ConstructorDeclaration;
export declare type Type = TypeReference | UnionType | IntersectionType | PrimitiveType | ObjectType | TypeofReference | FunctionType | TypeParameter | ThisType;
export declare type Import = ImportAllDeclaration | ImportDefaultDeclaration | ImportNamedDeclaration | ImportEqualsDeclaration | ImportDeclaration;
export declare type NamespaceMember = InterfaceDeclaration | TypeAliasDeclaration | ClassDeclaration | NamespaceDeclaration | ConstDeclaration | VariableDeclaration | FunctionDeclaration | CommentDeclaration | JsDocCommentDeclaration;
export declare type ModuleMember = InterfaceDeclaration | TypeAliasDeclaration | ClassDeclaration | NamespaceDeclaration | ConstDeclaration | VariableDeclaration | FunctionDeclaration | Import | ExportEqualsDeclaration | ExportDefaultDeclaration;
export declare type TopLevelDeclaration = NamespaceMember | ExportEqualsDeclaration | ExportDefaultDeclaration | ExportNameDeclaration | ModuleDeclaration | EnumDeclaration | Import;
export declare enum DeclarationFlags {
    None = 0,
    Private = 1,
    Protected = 2,
    Static = 4,
    Optional = 8,
    Export = 16,
    Abstract = 32,
    ExportDefault = 64,
    ReadOnly = 128
}
export declare enum ParameterFlags {
    None = 0,
    Optional = 1,
    Rest = 2
}
export declare enum CommentFlags {
    Wrap = 0,
    Plain = 1
}
export declare const config: {
    indentSpace: number;
    outputEol: string;
};
export declare const util: {
    getFullName(type: NamedDeclarationBase): any;
    typeToDeclaration(name: string, d: Type, flags?: DeclarationFlags): DeclarationBase | JsDocCommentDeclaration | CommentDeclaration;
    isNamedDeclarationBase(node: any): node is NamedDeclarationBase;
    isTypeofReference(node: any): node is TypeofReference;
    isFunctionType(node: any): node is FunctionType;
    isInterfaceDeclaration(node: any): node is InterfaceDeclaration;
    isNamespaceDeclaration(node: any): node is NamespaceDeclaration;
    isClassDeclaration(node: any): node is ClassDeclaration;
    isObjectType(node: any): node is ObjectType;
    isNamedTypeReference(node: any): node is NamedTypeReference;
    isConstructorDeclaration(node: any): node is ConstructorDeclaration;
    isImport(node: any): node is Import;
    isImportAllDeclaration(node: any): node is ImportAllDeclaration;
    isImportDefaultDeclaration(node: any): node is ImportDefaultDeclaration;
    isImportNamedDeclaration(node: any): node is ImportNamedDeclaration;
    isImportEqualsDeclaration(node: any): node is ImportEqualsDeclaration;
    isImportDeclaration(node: any): node is ImportDeclaration;
    isCallSignature(node: any): node is CallSignature;
    isConstDeclaration(node: any): node is ConstDeclaration;
    isFunctionDeclaration(node: any): node is FunctionDeclaration;
    isNamespaceMember(node: any): node is NamespaceMember;
    isCommentDeclaration(node: any): node is CommentDeclaration;
    isJsDocCommentDeclaration(node: any): node is JsDocCommentDeclaration;
    isTypeAliasDeclaration(node: any): node is TypeAliasDeclaration;
    isVariableDeclaration(node: any): node is VariableDeclaration;
    isExportEqualsDeclaration(node: any): node is ExportEqualsDeclaration;
    isExportDefaultDeclaration(node: any): node is ExportDefaultDeclaration;
    isExportNameDeclaration(node: any): node is ExportNameDeclaration;
    isModuleDeclaration(node: any): node is ModuleDeclaration;
    isEnumDeclaration(node: any): node is EnumDeclaration;
    isTopLevelDeclaration(node: any): node is TopLevelDeclaration;
    isCanBeExportDefault(node: any): boolean;
};
export declare const create: {
    comment(comment: string, flags?: CommentFlags | undefined): CommentDeclaration;
    jsDocComment(jsDocComment: string, flags?: CommentFlags | undefined): JsDocCommentDeclaration;
    interface(name: string, flags?: DeclarationFlags): InterfaceDeclaration;
    class(name: string, flags?: DeclarationFlags): ClassDeclaration;
    typeParameter(name: string, baseType?: InterfaceDeclaration | ClassDeclaration | NamedTypeReference | TypeParameter | undefined): TypeParameter;
    enum(name: string, constant?: boolean, flags?: DeclarationFlags): EnumDeclaration;
    enumValue(name: string, value?: string | number | undefined): EnumMemberDeclaration;
    property(name: string, type: Type, flags?: DeclarationFlags): PropertyDeclaration;
    method(name: string, parameters: Parameter[], returnType: Type, flags?: DeclarationFlags): MethodDeclaration;
    callSignature(parameters: Parameter[], returnType: Type): CallSignature;
    function(name: string, parameters: Parameter[], returnType: Type, flags?: DeclarationFlags): FunctionDeclaration;
    functionType(parameters: Parameter[], returnType: Type): FunctionType;
    parameter(name: string, type: Type, flags?: ParameterFlags): Parameter;
    constructor(parameters: Parameter[], flags?: DeclarationFlags): ConstructorDeclaration;
    const(name: string, type: Type, flags?: DeclarationFlags): ConstDeclaration;
    variable(name: string, type: Type): VariableDeclaration;
    alias(name: string, type: Type, flags?: DeclarationFlags): TypeAliasDeclaration;
    namespace(name: string): NamespaceDeclaration;
    objectType(members: ObjectTypeMember[]): ObjectType;
    indexSignature(name: string, indexType: "string" | "number", valueType: Type): IndexSignature;
    array(type: Type): ArrayTypeReference;
    namedTypeReference(name: string | NamedDeclarationBase): NamedTypeReference;
    exportEquals(target: string): ExportEqualsDeclaration;
    exportDefault(name: string): ExportDefaultDeclaration;
    exportName(name: string, as?: string | undefined): ExportNameDeclaration;
    module(name: string): ModuleDeclaration;
    importAll(name: string, from: string): ImportAllDeclaration;
    importDefault(name: string, from: string): ImportDefaultDeclaration;
    importNamed(names: {
        name: string;
        as?: string | undefined;
    }[], from: string): ImportNamedDeclaration;
    importEquals(name: string, from: string): ImportEqualsDeclaration;
    import(from: string): ImportDeclaration;
    union(members: Type[]): UnionType;
    intersection(members: Type[]): IntersectionType;
    typeof(type: NamedTypeReference): TypeofReference;
    tripleSlashReferencePathDirective(path: string): TripleSlashReferencePathDirective;
    tripleSlashReferenceTypesDirective(types: string): TripleSlashReferenceTypesDirective;
    tripleSlashReferenceNoDefaultLibDirective(value?: boolean): TripleSlashReferenceNoDefaultLibDirective;
    tripleSlashAmdModuleDirective(name?: string | undefined): TripleSlashAmdModuleDirective;
};
export declare const type: {
    array(type: Type): ArrayTypeReference;
    stringLiteral(string: string): PrimitiveType;
    numberLiteral(number: number): PrimitiveType;
    string: PrimitiveType;
    number: PrimitiveType;
    boolean: PrimitiveType;
    any: PrimitiveType;
    void: PrimitiveType;
    object: PrimitiveType;
    null: PrimitiveType;
    undefined: PrimitiveType;
    true: PrimitiveType;
    false: PrimitiveType;
    this: "this";
};
export declare const reservedWords: string[];
/** IdentifierName can be written as unquoted property names, but may be reserved words. */
export declare function isIdentifierName(s: string): boolean;
/** Identifiers are e.g. legal variable names. They may not be reserved words */
export declare function isIdentifier(s: string): boolean;
export declare enum ContextFlags {
    None = 0,
    Module = 1,
    InAmbientNamespace = 2
}
export declare function never(_x: never, err: string): never;
export interface EmitOptions {
    rootFlags?: ContextFlags;
    tripleSlashDirectives?: TripleSlashDirective[];
    indent?: number;
}
export declare function getWriter(rootDecl: any, { rootFlags, tripleSlashDirectives, indent, }?: EmitOptions): {
    writeAlias: (a: TypeAliasDeclaration) => void;
    writeClass: (c: ClassDeclaration) => void;
    writeClassMember: (c: ClassMember) => void;
    writeConst: (c: ConstDeclaration) => void;
    writeConstructorDeclaration: (ctor: ConstructorDeclaration) => void;
    writeDeclaration: (d: TopLevelDeclaration) => void;
    writeDelimited: <T>(arr: T[], sep: string | (() => void), printer: (x: T) => void) => void;
    writeEnum: (e: EnumDeclaration) => void;
    writeEnumValue: (e: EnumMemberDeclaration) => void;
    writeExportDefault: (e: ExportDefaultDeclaration) => void;
    writeExportEquals: (e: ExportEqualsDeclaration) => void;
    writeExportName: (e: ExportNameDeclaration) => void;
    writeFunction: (f: FunctionDeclaration) => void;
    writeFunctionType: (f: FunctionType) => void;
    writeImport: (i: ImportDeclaration) => void;
    writeImportAll: (i: ImportAllDeclaration) => void;
    writeImportDefault: (i: ImportDefaultDeclaration) => void;
    writeImportNamed: (i: ImportNamedDeclaration) => void;
    writeInterface: (d: InterfaceDeclaration) => void;
    writeMethodDeclaration: (m: MethodDeclaration) => void;
    writeModule: (m: ModuleDeclaration) => void;
    writeNamespace: (ns: NamespaceDeclaration) => void;
    writeParameter: (p: Parameter) => void;
    writeReference: (d: Type) => void;
    writeTripleSlashDirective: (t: TripleSlashDirective) => void;
    writeTypeParameters: (params: TypeParameter[]) => void;
    writeUnionReference: (d: Type) => void;
    writeVar: (c: VariableDeclaration) => void;
    readonly output: string;
};
export declare function emit(rootDecl: TopLevelDeclaration | TopLevelDeclaration[], options?: EmitOptions): string;
//# sourceMappingURL=dom.d.ts.map