import ts from 'typescript';
import * as util from './util';
import * as dom from './dom';
export { dom, util };
export interface PlainObj<T = any> {
    [key: string]: T;
}
export interface Declaration {
    import: dom.Import[];
    export: dom.TopLevelDeclaration[];
    fragment: dom.NamespaceMember[];
}
export interface ImportCacheElement {
    default?: string;
    list: Array<{
        name: string;
        as?: string;
    }>;
    from: string;
    realPath: string;
}
export interface CreateOptions {
    dist?: string;
    flags?: CreateDtsFlags;
}
export declare enum ExportFlags {
    None = 0,
    ExportEqual = 1,
    Export = 2
}
export interface Env {
    file: string;
    dist: string;
    flags: CreateDtsFlags;
    program: ts.Program;
    checker: ts.TypeChecker;
    sourceFile: ts.SourceFile;
    declaration: Declaration;
    importCache: {
        [key: string]: ImportCacheElement;
    };
    declareCache: Array<{
        node: ts.Declaration;
        name: string;
    }>;
    publicNames: PlainObj<number>;
    exportDefaultName?: string;
    exportNamespace?: dom.NamespaceDeclaration;
    interfaceList: dom.InterfaceDeclaration[];
    deps: {
        [key: string]: {
            env: Env;
            namespace: dom.NamespaceDeclaration;
        };
    };
    ambientModNames: string[];
    exportFlags: ExportFlags;
    toString: () => string;
    write: () => string;
}
export declare enum GetTypeDomFlags {
    None = 0,
    TypeLiteralInline = 1
}
export declare enum CreateDtsFlags {
    None = 0,
    IgnorePrivate = 1
}
export declare const defaultBuildFlags: number;
export interface ExportListObj {
    name: string;
    node: ts.Node;
    originalNode: ts.Node;
}
export declare function getTypeDom(typeNode?: ts.Node, flags?: GetTypeDomFlags): any;
export declare function getCallSignatureDeclarationTypeDom(node: ts.CallSignatureDeclaration): dom.CallSignature;
export declare function getLiteralTypeDom(typeNode: ts.LiteralTypeNode): any;
export declare function getFunctionTypeDom(typeNode: ts.FunctionTypeNode): any;
export declare function getTypeQueryTypeDom(typeNode: ts.TypeQueryNode): "string" | "number" | "boolean" | "undefined" | "object" | "any" | "void" | "null" | "true" | "false" | dom.StringLiteral | dom.NumberLiteral | dom.TypeofReference;
export declare function getTypeLiteralTypeDom(typeNode: ts.TypeLiteralNode, flags?: GetTypeDomFlags): dom.NamedTypeReference | dom.ObjectType;
export declare function getIntersectionTypeDom(typeNode: ts.IntersectionTypeNode): any;
export declare function getUnionTypeDom(typeNode: ts.UnionTypeNode): any;
export declare function createDepsByFile(file: string): {
    env: Env;
    namespace: dom.NamespaceDeclaration;
};
export declare function getImportTypeDom(typeNode: ts.ImportTypeNode): "string" | "number" | "boolean" | "undefined" | "object" | dom.NamedTypeReference | "any" | "void" | "null" | "true" | "false" | dom.StringLiteral | dom.NumberLiteral | dom.TypeofReference;
export declare function getReturnTypeFromDeclaration(declaration: ts.SignatureDeclaration): ts.TypeNode | undefined;
export declare function createClassDeclaration(className: string, node: ts.ClassLikeDeclaration): dom.ClassDeclaration;
export declare function findAssignToThis(statements: ts.NodeArray<ts.Statement>): {
    name: string;
    node?: ts.TypeNode | undefined;
}[];
export declare function addJsDocToTypeDom(typeDom: dom.DeclarationBase, originalNode: ts.Node): void;
export declare function eachPropertiesTypeDom<T extends ts.ClassElement | ts.ObjectLiteralElement>(nodeList: ts.NodeArray<T>, callback: (propName: string, d: T) => void): string[];
export declare function getPropTypeDomByNode(name: string, node?: ts.Node, flags?: dom.DeclarationFlags): dom.ClassMember;
export declare function getReferenceModule(symbol: ts.Symbol): false | ts.Declaration | undefined;
export declare function getReferenceTypeDomFromEntity(node: ts.EntityName): dom.NamedTypeReference | undefined;
export declare function createDeclarationTypeDom(name: string, node: ts.Declaration): dom.JsDocCommentDeclaration | dom.CommentDeclaration | dom.NamespaceDeclaration | dom.InterfaceDeclaration | dom.TypeAliasDeclaration | dom.ClassDeclaration | dom.ConstDeclaration | dom.VariableDeclaration | dom.FunctionDeclaration | undefined;
export declare function getPropertyAccessTypeDom(node: ts.PropertyAccessExpression): any;
export declare function getReferenceTypeDom(typeNode: ts.TypeReferenceNode): "string" | "number" | "boolean" | "undefined" | "object" | dom.NamedTypeReference | "any" | "void" | "null" | "true" | "false" | dom.StringLiteral | dom.NumberLiteral;
export declare function getArrayTypeDom(typeNode: ts.ArrayTypeNode): any;
export declare function getFunctionParametersTypeDom(parameters: ts.NodeArray<ts.ParameterDeclaration>): dom.Parameter[];
export declare function checkIsPrivate(obj: ts.Node, name?: string): boolean | "" | undefined;
export declare function addMemberToObj(obj: dom.ClassDeclaration | dom.InterfaceDeclaration | dom.ObjectType, member: dom.ClassMember | dom.ObjectTypeMember, opt?: {
    preInsert?: boolean;
    dropDuplicate?: boolean;
}): void;
export declare function tryCreateFunctionAsClass(fnName: string, node: ts.FunctionDeclaration): dom.ClassDeclaration | undefined;
export declare function createFunctionDeclaration(fnName: string, node: ts.FunctionLike): any;
export declare function getModNameByPath(fileName: string): string | undefined;
export declare function getTypeNodeAtLocation(node: ts.Node, flag?: ts.NodeBuilderFlags): ts.TypeNode | undefined;
export declare function createVariableDeclaration(constName: string, node: ts.VariableDeclaration): dom.ConstDeclaration;
export declare function getDeclName(name: string): string;
export declare function createInterfaceWithCache(members: dom.ObjectTypeMember[]): dom.InterfaceDeclaration;
export declare function createExportInterface(name: string): dom.InterfaceDeclaration;
export declare function createExportNameByFile(file: string): string;
export declare function create(file: string, options?: CreateOptions): Env;
//# sourceMappingURL=index.d.ts.map